import arcade
import arcade.gui
from game import GameScreen
from constants import SCREEN_WIDTH, SCREEN_HEIGHT


class QuitButton(arcade.gui.UIFlatButton):
    def on_click(self, event: arcade.gui.UIOnClickEvent):
        arcade.exit()


class MenuScreen(arcade.View):
    def __init__(self):
        super().__init__()
        arcade.set_viewport(0, SCREEN_WIDTH, 0, SCREEN_HEIGHT)

        self.manager = arcade.gui.UIManager()
        self.manager.enable()

        # Set background color
        arcade.set_background_color(arcade.color.DARK_BLUE_GRAY)

        # Alignment boxes
        self.v_box = arcade.gui.UIBoxLayout()
        self.h_box = arcade.gui.UIBoxLayout(vertical=False)

    def on_show_view(self):
        # Title
        game_title = arcade.gui.UITextArea(text="Wizard Detention",
                                           width=500,
                                           height=40,
                                           font_size=30,
                                           font_name="Kenney Future")
        self.v_box.add(game_title.with_space_around(bottom=50))

        # Start button
        start_button = arcade.gui.UIFlatButton(text="Start", width=200)
        self.h_box.add(start_button.with_space_around(right=20))
        start_button.on_click = self.on_click_start

        # Quit button
        quit_button = QuitButton(text="Quit", width=200)
        self.h_box.add(quit_button.with_space_around(left=20))

        self.v_box.add(self.h_box.with_space_around(bottom=20))

        # Credits button
        credits_button = arcade.gui.UIFlatButton(text="Credits", width=100)
        self.v_box.add(credits_button.with_space_around(bottom=20))
        credits_button.on_click = self.on_click_credits

        self.manager.add(
            arcade.gui.UIAnchorWidget(
                anchor_x="center_x",
                anchor_y="center_y",
                child=self.v_box)
        )

    def on_click_start(self, event):
        game_screen = GameScreen()
        self.window.show_view(game_screen)

    def on_click_credits(self, event):
        message_box = arcade.gui.UIMessageBox(
            width=SCREEN_WIDTH / 2,
            height=SCREEN_HEIGHT / 2,
            message_text=(
                "Credits Screen\n"
                "Game made by Julia McKay, Chandler Fox, and Audrey DeHoog"
            ),
            callback=self.on_message_box_close,
            buttons=["Back"]
        )
        self.manager.add(message_box)

    @staticmethod
    def on_message_box_close(button_text):
        print(f"User pressed {button_text}.")

    def on_draw(self):
        self.clear()
        self.manager.draw()


import arcade
import arcade.gui
from command import *
from constants import *


class GameScreen(arcade.View):
    def __init__(self):
        """Create the Window here and Declare game variables"""
        # Create the ViewPort
        super().__init__()
        arcade.set_viewport(0, SCREEN_WIDTH, 0, SCREEN_HEIGHT)

        # Create the manager
        self.manager = arcade.gui.UIManager()
        self.manager.enable()

        # Variable Declarations
        self.ih = None

        # Game screen
        self.tile_map = None
        self.scene = None

        # Player Sprites
        self.wizard_sprite = None
        self.familiar_sprite = None

        #Moving Platform Sprites
        self.moving_platform_1 = None
        self.moving_vel = 2
        self.move_plat_1 = True

        # Physics Engine
        self.pe1 = None
        self.pe2 = None
        self.ty = None

    def setup(self):
        """Set up the game here. Call this function to restart the game."""
        # name of map to load
        map_name = "Assets\\Maps\\Level_0_map.json"
        layer_options = {
            "Platforms": {
                "use_spatial_hash": True,
            },
            "Dont Touch": {
                "use_spatial_hash": True,
            },
        }

        self.tile_map = arcade.load_tilemap(map_name, TILE_SCALING, layer_options)
        self.scene = arcade.Scene.from_tilemap(self.tile_map)

        #Adding Moving Platform Sprite
        self.moving_platform_1 = arcade.Sprite("Assets\\Sprites\\moving_platform_01.png", PLATFORM_SCALING)
        self.moving_platform_1.center_x = 1175
        self.moving_platform_1.center_y = 380
        self.scene.add_sprite("Platforms", self.moving_platform_1)

        # Player Sprite Setup
        self.scene.add_sprite_list("Wiz")
        self.scene.add_sprite_list("Cat")
        self.scene.add_sprite_list("Walls", use_spatial_hash=True)

        self.wizard_sprite = arcade.Sprite("Assets\\Sprites\\R_witch_stationary.png", WIZARD_SCALING)
        self.wizard_sprite.position = (SPAWN_X, SPAWN_Y)
        self.scene.add_sprite("Wiz", self.wizard_sprite)

        self.familiar_sprite = arcade.Sprite("Assets\\Sprites\\cat05.png", FAMILIAR_SCALING)
        self.familiar_sprite.position = (SPAWN_X + 30, SPAWN_Y - 10)
        self.scene.add_sprite("Cat", self.familiar_sprite)

        self.ih = InputHandler(self.wizard_sprite, self.familiar_sprite)

        self.pe1 = arcade.PhysicsEnginePlatformer(self.wizard_sprite, gravity_constant=GRAVITY,
                                                  walls=self.scene["Platforms"])
        self.pe2 = arcade.PhysicsEnginePlatformer(self.familiar_sprite, gravity_constant=GRAVITY,
                                                  walls=self.scene["Platforms"])

    def on_show_view(self):
        self.setup()
        arcade.set_background_color(arcade.color.GRAY)

    def on_draw(self):
        self.clear()
        self.manager.draw()
        self.scene.draw()

    def on_key_press(self, key, mods):
        # Check command.py for commands
        command = self.ih.handle_input(key)
        if command:
            command()

    def on_key_release(self, key, mods):
        command = self.ih.handle_input(key)
        if command:
            command.undo()

    def on_update(self, delta_time):
        self.pe1.update()
        self.pe2.update()

        #If lever is pressed, move platform
        if self.move_plat_1:
            # Adding moving velocity in moving platform
            self.moving_platform_1.center_y -= self.moving_vel
            # Stop once you reach bottom
            if self.moving_platform_1.center_y < 250 or self.moving_platform_1.center_y > 380:
                self.move_plat_1 = False

        #See if player has collided w anything from the Dont Touch layer
        if arcade.check_for_collision_with_list(self.wizard_sprite, self.scene["Dont Touch"]):
            self.wizard_sprite.position = (SPAWN_X, SPAWN_Y)
        if arcade.check_for_collision_with_list(self.familiar_sprite, self.scene["Dont Touch"]):
            self.familiar_sprite.position = (SPAWN_X + 30, SPAWN_Y - 10)



import arcade
from menu_screen import MenuScreen
from constants import SCREEN_WIDTH, SCREEN_HEIGHT


def main():
    """Main function"""
    window = arcade.Window(SCREEN_WIDTH, SCREEN_HEIGHT, "Wizard Detention")
    start_view = MenuScreen()
    window.show_view(start_view)
    arcade.run()


if __name__ == "__main__":
    main()



from abc import abstractmethod
import arcade
from constants import PLAYER_JS, PLAYER_MS


class InputHandler:
    """Handles Input based off a given key press"""
    def __init__(self, wiz: arcade.Sprite, cat: arcade.Sprite):
        """To add a command, add the key to press to the list as well and assign the correct object constructor"""
        self.commands = \
            {
                arcade.key.A: MoveLeftCommand(wiz),
                arcade.key.D: MoveRightCommand(wiz),
                arcade.key.W: JumpCommand(wiz),
                arcade.key.LEFT: MoveLeftCommand(cat),
                arcade.key.RIGHT: MoveRightCommand(cat),
                arcade.key.UP: JumpCommand(cat)
            }

    def handle_input(self, key_pressed):
        """Simply picks the command"""
        if key_pressed in self.commands:
            return self.commands[key_pressed]
        return None


class Command:
    """An abstract class used for all command classes"""
    def __init__(self, sprite: arcade.Sprite):
        self.sprite = sprite

    @abstractmethod
    def __call__(self):
        pass

    @abstractmethod
    def undo(self):
        pass


class JumpCommand(Command):
    """Makes the character sprite jump"""
    def __init__(self, sprite: arcade.Sprite):
        self.sprite = sprite

    def __call__(self):
        self.sprite.change_y = PLAYER_JS

    def undo(self):
        self.sprite.change_y = 0


class MoveLeftCommand(Command):
    """Makes the character sprite move left"""
    def __init__(self, sprite: arcade.Sprite):
        self.sprite = sprite

    def __call__(self):
        self.sprite.change_x = -PLAYER_MS

    def undo(self):
        self.sprite.change_x = 0


class MoveRightCommand(Command):
    """Makes the character sprite move right"""
    def __init__(self, sprite: arcade.Sprite):
        self.sprite = sprite

    def __call__(self):
        self.sprite.change_x = PLAYER_MS

    def undo(self):
        self.sprite.change_x = 0



# Constants
SCREEN_WIDTH = 1280
SCREEN_HEIGHT = 720
SCREEN_TITLE = "Wizard Detention"

TILE_SCALING = 1
WIZARD_SCALING = 1
FAMILIAR_SCALING = 1
PLATFORM_SCALING = 1.8

SPAWN_X = 100
SPAWN_Y = 120

PLAYER_MS = 5
PLAYER_JS = 15
GRAVITY = 1
